@@ -31,6 +31,9 @@ MENU_CLOSE_ALL = wx.NewId()
 MENU_CLOSE_WINDOW = wx.NewId()
 MENU_FILE_SAVE = wx.NewId()
 MENU_FILE_SAVE_TABLE = wx.NewId()
+MENU_INTERPOLATION_NEAREST = wx.NewId()
+MENU_INTERPOLATION_BILINEAR = wx.NewId()
+MENU_INTERPOLATION_BICUBIC = wx.NewId()
 MENU_LABELS_ALPHA = {}
 MENU_LABELS_LINES = {}
 MENU_LABELS_OFF = {}
