@@ -112,7 +112,7 @@ FF_CENTROID = "%s_%s_%%s" % (C_DISTANCE, FEAT_CENTROID)
 """Minimum distance measurement (FF_MINIMUM % parent)"""
 FF_MINIMUM = "%s_%s_%%s" % (C_DISTANCE, FEAT_MINIMUM)
 
-FIXED_SETTING_COUNT = 5
+FIXED_SETTING_COUNT = 7
 VARIABLE_SETTING_COUNT = 1
 
 
@@ -916,7 +916,7 @@ parents or children of the parent object.""",
 
                 return measurements
             elif category == "Children":
-                return ["{}_Count".format(self.y_name.value)]
+                return ["%s_Count" % self.y_name.value]
         elif object_name == self.y_name.value and category == "Parent":
             return [self.x_name.value]
         elif object_name == self.y_name.value and category == C_DISTANCE:
